import { select, json, geoPath, geoNaturalEarth1 } from 'd3';
import { feature } from 'topojson';


const query = `PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX dc: <http://purl.org/dc/elements/1.1/>
PREFIX dct: <http://purl.org/dc/terms/>
PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
PREFIX edm: <http://www.europeana.eu/schemas/edm/>
PREFIX foaf: <http://xmlns.com/foaf/0.1/>
PREFIX hdlh: <https://hdl.handle.net/20.500.11840/termmaster>
PREFIX wgs84: <http://www.w3.org/2003/01/geo/wgs84_pos#>
PREFIX geo: <http://www.opengis.net/ont/geosparql#>
PREFIX skos: <http://www.w3.org/2004/02/skos/core#>
PREFIX gn: <http://www.geonames.org/ontology#>
PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#>

SELECT ?cho ?title ?typeLabel ?img ?placeName ?lat ?long WHERE {
  <https://hdl.handle.net/20.500.11840/termmaster1397> skos:narrower* ?type .
  ?type skos:prefLabel ?typeLabel .
  
  ?cho dc:title ?title .
  ?cho edm:isShownBy ?img .
  ?cho dct:spatial ?place .
  ?place skos:prefLabel ?placeName . 
  ?place skos:exactMatch/wgs84:lat ?lat .
  ?place skos:exactMatch/wgs84:long ?long .
  #?place skos:exactMatch/gn:parentCountry ?land .
  #?land gn:name ?landLabel .
  
}`
//Please use your own endpoint when using this 
const endpoint = "https://api.data.netwerkdigitaalerfgoed.nl/datasets/ivo/NMVW/services/NMVW-06/sparql"

const svg = select('svg')
const circleDelay = 10
const circleSize = 8
const projection = geoNaturalEarth1()
const pathGenerator = geoPath().projection(projection)

setupMap()
drawMap()
plotLocations()

function setupMap(){
  svg
    .append('path')
    .attr('class', 'sphere')
    .attr('d', pathGenerator({ type: 'Sphere' }))
}

function drawMap() {
  d3.json('https://unpkg.com/world-atlas@1.1.4/world/110m.json').then(data => {
    const countries = feature(data, data.objects.countries);
    svg
      .selectAll('path')
      .data(countries.features)
      .enter()
      .append('path')
      .attr('class', 'country')
      .attr('d', pathGenerator)
  })
}

function plotLocations() {
  fetch(endpoint +"?query="+ encodeURIComponent(query) + "&format=json")
    .then(data => data.json())
  	.then(json => json.results.bindings)
    .then(results => {
    //TODO: clean up results in separate function
    	results.forEach(result => {
        result.lat = Number(result.lat.value)
        result.long = Number(result.long.value)
      })    
    	console.log(results)
      
    svg
        .selectAll('circle')
        .data(results)
        .enter()
        .append('circle')
        .attr('class', 'circles')
        .attr('cx', function(d) {
          return projection([d.long, d.lat])[0]
        })
        .attr('cy', function(d) {
          return projection([d.long, d.lat])[1]
        })
        //.attr('r', '0px')
    		//Opacity is quite heavy on the rendering process so I've turned it off	
    		.attr('opacity', .5)
        .transition()
        .delay(function(d, i) { return i * circleDelay; })
            .duration(1000)
            .ease(d3.easeBounce)
            .attr('r', circleSize+'px')
  })
}
